% Summary:
%   Script that simulates AGC scaling and ADC quantization on real data 
%   recorded on a binary file.
%
% Output:
%   A binary file with quantized data and pulse interference removed
%
% History:
%   Jun 28/12 - Script created by Daniele Borio.
clear all
close all
clc

% Call 'readOption' for reading the processing options
readOption;

% The following variables should have been loaded

% inputFile - the name of the input file
% outputFile - the name of the output file
% fs - the sampling frequency
% fif - the intermediate frequency (can be used by IC)
% IsComplex - true if the input file is in I/Q format 
% Is16Bits - true if the input samples are on 16 bits
% nbits - number of bits
% Tt - processing time, i.e., how many samples of data should be processed
% skipBytes - number of bytes to skip
% ending - ending of the input file

% Open the input file
fidIn = fopen( inputFile, 'r' , ending );
% ... eventually skip some bytes
fseek( fidIn, skipBytes, 'bof');

% Open the output file
fidOut = fopen( outputFile, 'w', ending );

% The processing will be done epoch by epoch:
% define the epoch parameters
Te = 0.01;  % process 10 ms of data per epoch
blksize = round( Te * fs );  % number of samples for epoch

Nepoch = round( Tt / Te );  % number of processing epochs
sigma2 = 0; % Initial estimate of the noise variance
AgcValues = zeros( 1, length( Nepoch ) );

% MAIN PROCESSING LOOP
for ii = 1:Nepoch,
    % Display the status 
    if ( rem( ii * Te, 1 ) == 0 )   % each second
        disp( sprintf('%d seconds processed over %d', ii*Te, Tt ));
    end
    
    % Read data from file
    if Is16Bits,
        if IsComplex,
            inputData = fread( fidIn, 2 * blksize, 'int16' );
            inputData = inputData( 1:2:end ).' + 1i * inputData( 2:2:end ).';
        else
            inputData =	fread( fidIn, blksize, 'int16' );
            inputData = inputData.';
        end
    else
         if IsComplex,
            inputData = fread( fidIn, 2 * blksize, 'int8' );
            inputData = inputData( 1:2:end ).' + 1i * inputData( 2:2:end ).';
        else
            inputData =	fread( fidIn, blksize, 'int8' );
            inputData = inputData.';
        end
    end
    
    if( length( inputData ) < blksize ),
        % Exit from the loop if there are not enough samples
        break;
    end  
    
    % Do the real processing
    % Quantization
    [ quantData sigma2 AgcValues( ii ) ] = Quantize( inputData, nbits, sigma2, IsComplex );
        
    % Save the data
    saveToFile( fidOut, quantData, blksize, IsComplex, Is16Bits );
    
end
% END PROCESSING LOOP

% Plot the AGC gain
time = ( 0:(Nepoch - 1) ) * Te;
plot( time, AgcValues, 'LineWidth', 2 )
grid on
axis tight;
set(gca, 'FontSize', 16 )
xlabel('Time [s]')
ylabel('AGC value')

% Close input and output files
fclose( fidIn );
fclose( fidOut );